#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/4/14 15:44
@File: utils.py
@IDE: pycharm
@Description:
    测试控制器中的工具
"""
import pytest
from swanlab.server.controller.utils import *
import random


class TestLTTB:
    @pytest.mark.parametrize("data, sample_length", [
        ([{"data": random.random(), "index": i} for i in range(100)], 100),
        ([{"data": random.random(), "index": i} for i in range(1500)], 1500),
        ([{"data": random.random(), "index": i} for i in range(1502)], 1502),
    ])
    def test_no_sample(self, data, sample_length):
        """
        测试数据长度小于等于采样长度
        """
        result = lttb(data)
        assert len(result) == len(data)

    @pytest.mark.parametrize("data, sample_length", [
        ([{"data": random.random(), "index": i} for i in range(1503)], 1500),
        ([{"data": random.random(), "index": i} for i in range(1600)], 1500),
        ([{"data": random.random(), "index": i} for i in range(5000)], 1500),
    ])
    def test_sample(self, data, sample_length):
        """
        测试数据长度大于采样长度
        """
        result = lttb(data)
        assert len(result) == sample_length



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-03-05 19:42:52
@File: swanlab/db/migrate/namespace_opened.py
@IDE: vscode
@Description:
    为namespace增加opened字段
"""
from peewee import SqliteDatabase, IntegerField
from playhouse.migrate import migrate, SqliteMigrator


def add_opened(db):
    """
    为数据库的namespace表添加opened字段,用于记录命名空间是否已经被打开
    默认值为False
    """
    migrator = SqliteMigrator(db)
    opened = IntegerField(default=1, choices=[0, 1])
    migrate(migrator.add_column("namespace", "opened", opened))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-03-09 17:43:27
@File: swanlab/db/migrate/project.py
@IDE: vscode
@Description:
    为数据库的Project表添加迁移操作
"""
from peewee import IntegerField
from playhouse.migrate import migrate, SqliteMigrator


def add_pinned_opened(db):
    """
    为数据库的Project表添加pinned_opened字段,用于记录项目置顶部分是否打开
    默认值为1，表示打开
    """
    migrator = SqliteMigrator(db)
    pinned_opened = IntegerField(default=1, choices=[0, 1])
    migrate(migrator.add_column("project", "pinned_opened", pinned_opened))


def add_hidden_opened(db):
    """
    为数据库的Project表添加hidden_opened字段,用于记录项目隐藏部分是否打开
    默认值为0,表示关闭
    """
    migrator = SqliteMigrator(db)
    hidden_opened = IntegerField(default=0, choices=[0, 1])
    migrate(migrator.add_column("project", "hidden_opened", hidden_opened))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-02-05 16:56:06
@File: swanlab/db/connect.py
@IDE: vscode
@Description:
    数据库连接模块
"""
from swanlab.env import get_db_path
import os
from peewee import SqliteDatabase
from .table_config import tables, Tag, Experiment, Namespace, Chart
from .migrate import *

# 判断是否已经binded了
binded = False


def connect(autocreate=False) -> SqliteDatabase:
    """
    连接数据库，只有调用此方法以后，数据库才会被创建，所有导出的类才可用
    这样设计的原因是因为路径问题，这里需要动态导入settings
    第一次调用此方法时会创建数据库，返回None，以后调用此方法会返回数据库实例，通过其自身的`connect`方法连接数据库
    完成复杂的数据库操作后，需要调用其`close`方法关闭数据库连接

    Parameters:
    ----------
    existed : bool
        是否指定创建数据库，如果为True，则会自动创建数据库，否则不会自动创建数据库
        设置为False时，如果目标数据库不存在，则会抛出FileNotFoundError异常
        但是即使设置为true，文件夹不存在的情况下也会抛出FileNotFoundError异常

    Return:
    -------
    swandb :
        数据库实例

    Raises:
    -------
    FileNotFoundError :
        如果数据库不存在且existed为False，则会抛出FileNotFoundError异常
        或者路径不存在，也会抛出FileNotFoundError异常
    """
    global binded
    path = get_db_path()
    # 判断数据库是否存在
    db_exists = os.path.exists(path)
    path_exists = os.path.exists(os.path.dirname(path))
    if not path_exists or (not db_exists and not autocreate):
        raise FileNotFoundError(f"DB file {path} not found")
    # 启用外键约束
    swandb = SqliteDatabase(path, pragmas={"foreign_keys": 1})
    if not binded:
        # 动态绑定数据库
        swandb.connect()
        swandb.bind(tables)
        swandb.create_tables(tables)
        swandb.close()
        # 完成数据迁移，如果chart表中没有status字段，则添加
        if not Chart.field_exists("status"):
            # 不启用外键约束
            add_status(SqliteDatabase(path))
        # 完成数据迁移，如果namespace表中没有opened字段，则添加
        if not Namespace.field_exists("opened"):
            # 不启用外键约束
            add_opened(SqliteDatabase(path))
        # 完成数据迁移，如果experiment表中没有finish_time字段，则添加
        if not Experiment.field_exists("finish_time"):
            # 不启用外键约束
            add_finish_time(SqliteDatabase(path))
        # 完成数据迁移，如果tag表中没有sort字段，则添加
        if not Tag.field_exists("sort"):
            # 不启用外键约束
            add_sort(SqliteDatabase(path))
        binded = True
    return swandb



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/4/3 16:52
@File: conftest.py.py
@IDE: pycharm
@Description:
    配置pytest
"""
import pytest
from tutils import clear, init_db, SWANLAB_DIR, SWANLAB_LOG_DIR, PACKAGE_PATH
import swanlab.env as E
import shutil
import os


@pytest.fixture(scope="session", autouse=True)
def setup_before_all():
    clear()
    init_db()


@pytest.fixture(scope="function", autouse=True)
def setup_before_each():
    E.reset_env()
    if os.path.exists(SWANLAB_DIR):
        shutil.rmtree(SWANLAB_DIR)
    yield
    E.reset_env()
    shutil.rmtree(SWANLAB_DIR, ignore_errors=True)
    os.environ[E.DEV] = "TRUE"
    os.environ[E.ROOT] = SWANLAB_LOG_DIR
    os.environ[E.PACKAGE] = PACKAGE_PATH



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-03-09 17:39:12
@File: swanlab/db/migrate/chart.py
@IDE: vscode
@Description:
    chart相关迁移操作
"""
from peewee import SqliteDatabase, IntegerField
from playhouse.migrate import migrate, SqliteMigrator
from .experiment import (
    add_hidden_opened as add_hidden_opened_experiment,
    add_pinned_opened as add_pinned_opened_experiment,
)
from .project import add_hidden_opened as add_hidden_opened_project, add_pinned_opened as add_pinned_opened_project


def add_status(db):
    """
    为数据库的Chart表添加status字段，默认为0
    """
    migrator = SqliteMigrator(db)
    status = IntegerField(default=0)
    migrate(migrator.add_column("chart", "status", status))
    # 同时，添加sort字段
    add_sort(db)
    # 同时为experiment和project添加hidden_opened、pinned_opened字段
    add_hidden_opened_experiment(db)
    add_pinned_opened_experiment(db)
    add_hidden_opened_project(db)
    add_pinned_opened_project(db)


def add_sort(db):
    """
    为数据库的Chart表添加sort字段，默认为NULL
    """
    migrator = SqliteMigrator(db)
    sort = IntegerField(default=None, null=True)
    migrate(migrator.add_column("chart", "sort", sort))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-02-19 15:40:58
@File: swanlab/db/migrate/tag_sort.py
@IDE: vscode
@Description:
    为数据库的Tag表添加sort字段
"""
from peewee import SqliteDatabase, IntegerField
from playhouse.migrate import migrate, SqliteMigrator


def add_sort(db: SqliteDatabase):
    """
    为数据库的Tag表添加sort字段
    """
    migrator = SqliteMigrator(db)
    sort = IntegerField(default=0)
    migrate(migrator.add_column("tag", "sort", sort))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/5/16 20:42
@File: create_error_chart.py
@IDE: pycharm
@Description:
    创建错误图表
"""
from tutils import open_dev_mode
import swanlab

swanlab.login(api_key=open_dev_mode())

swanlab.init(
    description="此实验的图表将出现问题",
    log_level="debug",
    mode="cloud",
)

for i in range(10):
    swanlab.log({"success": i, "error": "abc"})

for i in range(10):
    if i % 2 == 1:
        swanlab.log({"nan_test": "nan", "inf_test": "inf"})
    else:
        swanlab.log({"nan_test": i, "inf_test": i})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2023-11-24 21:12:52
@File: test/start_server.py
@IDE: vscode
@Description:
    开启后端测试服务，访问端口
    事实上在使用中并不是这样的，而是在命令行执行命令完成的，这里只是为了测试和开发
    增加了实际开发中不会用到的热启动功能
"""
import uvicorn
from tutils.config import SWANLAB_LOG_DIR
from swanboard.db import connect
from swanboard.app import app
import os

os.environ["SWANLAB_LOG_DIR"] = SWANLAB_LOG_DIR

connect()


def main():
    return app


if __name__ == "__main__":
    uvicorn.run(
        "start_server:main",
        host="0.0.0.0",
        port=6092,
        reload=True,
        log_level="critical",
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/5/16 17:38
@File: use_swanlog.py
@IDE: pycharm
@Description:
    使用swanlog打印日志
"""
from swanlab.log import swanlog

SHOW = "这条日志应该显示"
HIDE = "这条日志不应该显示"

# ---------------------------------- 初始情况 ----------------------------------

swanlog.debug(HIDE)
swanlog.info(SHOW)
swanlog.warning(SHOW)
swanlog.error(SHOW)
swanlog.critical(SHOW)

# ---------------------------------- 安装日志系统 ----------------------------------

swanlog.install(log_level="debug")

swanlog.debug(SHOW)

# ---------------------------------- 卸载日志系统 ----------------------------------

swanlog.uninstall()

swanlog.debug(HIDE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-04-03 00:08:50
@File: test/unit/pytest_example.py
@IDE: vscode
@Description:
    测试用例示例，所有测试用例文件以pytest_开头
"""


def add(x, y):
    """简单的加法函数"""
    return x + y


def test_addition():
    """测试加法函数"""
    assert add(3, 5) == 8
    assert add(-1, 1) == 0
    assert add(2.5, 2.5) == 5


def subtract(x, y):
    """简单的减法函数"""
    return x - y


def test_subtraction():
    """测试减法函数"""
    assert subtract(5, 3) == 2
    assert subtract(1, -1) == 2
    assert subtract(2.5, 1) == 1.5



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-04-02 21:23:30
@File: swanlab/compat/server/controller/experiment.py
@IDE: vscode
@Description:
    用于兼容实验部分的接口
"""
import shutil
from ....db import *
from urllib.parse import unquote
from swanboard.settings import (
    get_media_dir,
    get_tag_dir,
)
import os
import ujson


def _transfer_logs(log_path: str, text_path: str):
    """
    将原本存于某 txt 中的内容直接以字符串的形式存到对应的 log 文件
    一个 log 文件可能对应多个 txt 文件，该函数一次处理一个 log 文件

    Parameters
    ----------
    log_path : str
        tag 对应的 logs 下目录的路径
    text_path : str
        tag 对应 media 下目录的路径

    Returns
    -------
        单个文件转化后的数据
    """
    with open(log_path, "r") as f:
        data = f.read().split("\n")
        # 将每个元素转成dict
        data = [ujson.loads(line) for line in data if line]
    # 打开对应的 txt 文件
    for index, log in enumerate(data):
        path = os.path.join(text_path, log["data"])
        with open(path, "r") as f:
            text = f.read()
        data[index]["data"] = text
    # # 保存 log 文件
    with open(log_path, "w") as f:
        for line in data:
            f.write(ujson.dumps(line, ensure_ascii=False) + "\n")
    return data


def compat_text(experiment_id: int, tag: str):
    """
    0.3.0 之前，text 图表中的媒体数据单独存放于 txt 文件中
    但是因为其本质是字符串，可以直接存储于 logs 下

    Parameters
    ----------
    experiment_id : int
        实验 id
    tag : str
        tag 名

    Returns
    -------
        如果不需要转化，返回 False，如果需要转化，返回转化后的数据
    """

    tag = Tag.filter(
        Tag.experiment_id == experiment_id, Tag.name == unquote(tag)
    ).first()
    if tag.type != "text":
        return False
    # 检测在 media 下是否存在同名目录
    run_id = Experiment.get(experiment_id).run_id
    text_path = get_media_dir(run_id, tag.name)
    # 是否存在该路径
    if not os.path.exists(text_path):
        return False
    # 将文件内容复制到 logs 记录中
    tag_path = get_tag_dir(run_id, tag.name)
    # 列出下面所有文件，除了 _summary.json
    files = [file for file in os.listdir(tag_path) if file != "_summary.json"]
    data: list = []
    for file in files:
        data.extend(_transfer_logs(os.path.join(tag_path, file), text_path))
    # 删除原来的媒体目录
    shutil.rmtree(text_path)
    return data
